import { KeyName } from './keyname';
import { Constants } from './constants';
export declare enum TargetDevice {
    Monochrome = 1,
    RGB = 2,
    PerKeyRGB = 4,
    All = 7
}
export declare enum DeviceType {
    Keyboard = 0,
    Mouse = 3,
    Mousemat = 4,
    Headset = 8,
    Speaker = 14
}
export interface SdkVersion {
    majorNum?: number;
    minorNum?: number;
    buildNum?: number;
}
export interface TargetDeviceSetting {
    targetDevice: TargetDevice;
}
export interface ColorSetting {
    redPercentage: number;
    greenPercentage: number;
    bluePercentage: number;
}
export interface DeviceTypeSetting {
    deviceType: DeviceType;
}
export interface EffectTimingSetting {
    milliSecondsDuration: number;
    milliSecondsInterval: number;
}
export interface KeyNameSetting {
    keyName: KeyName;
}
export interface KeyCodeSetting {
    keyCode: number;
}
export interface KeyListSetting {
    keyList: Array<KeyName>;
}
export interface BitmapSetting {
    bitmap: Uint8Array;
}
interface ConfigSetting<T> {
    configPath: string;
    defaultValue: T;
    value?: T;
}
export declare type ConfigNumberSetting = ConfigSetting<number>;
export declare type ConfigBooleanSetting = ConfigSetting<boolean>;
export interface ConfigLabelSetting {
    configPath: string;
    label: string;
}
export interface ConfigColorSetting {
    configPath: string;
    defaultRed: number;
    defaultGreen: number;
    defaultBlue: number;
    red?: number;
    green?: number;
    blue?: number;
}
export interface PulseKeySetting {
    startRedPercentage: number;
    startGreenPercentage: number;
    startBluePercentage: number;
    finishRedPercentage: number;
    finishGreenPercentage: number;
    finishBluePercentage: number;
    milliSecondsDuration: number;
    isInfinite: boolean;
}
export interface LogiLed {
    init(): boolean;
    getSdkVersion(versionOut: SdkVersion): boolean;
    getConfigOptionNumber(options: ConfigNumberSetting): boolean;
    getConfigOptionBool(options: ConfigBooleanSetting): boolean;
    getConfigOptionColor(options: ConfigColorSetting): boolean;
    setConfigOptionLabel(options: ConfigLabelSetting): boolean;
    setTargetDevice(options: TargetDeviceSetting): boolean;
    saveCurrentLighting(): boolean;
    setLighting(options: ColorSetting): boolean;
    restoreLighting(): boolean;
    flashLighting(options: ColorSetting & EffectTimingSetting): boolean;
    pulseLighting(options: ColorSetting & EffectTimingSetting): boolean;
    stopEffects(): boolean;
    setLightingFromBitmap(options: BitmapSetting): boolean;
    setLightingForKeyWithScanCode(options: KeyCodeSetting & ColorSetting): boolean;
    setLightingForKeyWithHidCode(options: KeyCodeSetting & ColorSetting): boolean;
    setLightingForKeyWithQuartzCode(options: KeyCodeSetting & ColorSetting): boolean;
    setLightingForKeyWithKeyName(options: KeyNameSetting & ColorSetting): boolean;
    saveLightingForKey(options: KeyNameSetting): boolean;
    restoreLightingForKey(options: KeyNameSetting): boolean;
    excludeKeysFromBitmap(options: KeyListSetting): boolean;
    flashSingleKey(options: KeyNameSetting & ColorSetting & EffectTimingSetting): boolean;
    pulseSingleKey(options: KeyNameSetting & PulseKeySetting): boolean;
    stopEffectsOnKey(options: KeyNameSetting): boolean;
    shutdown(): void;
}
export declare type LogiLedStatic = LogiLed & typeof Constants & {
    KeyName: typeof KeyName;
    DeviceType: typeof DeviceType;
    TargetDevice: typeof TargetDevice;
};
export {};
